/*
 * mp3.c
 * Copyright (C) 2000 David Podeur for convergence integrated media GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 *
 * The author can be reached at david@convergence.de, 

 * the project's page is at http://linuxtv.org/mbone/
 */
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>

#define BUFFER_SIZE           50000
#define MP3_HEADER_SIZE       4
#define MAX_MP3_FRAME_LENGTH  1000
#define MIN_MP3_FRAME_LENGTH  300
#define MAX_RTP_SIZE          24
#define WARNINGS              1


/****************************
 * MP3 related structures   *
 ***************************/

typedef struct {
	unsigned char data[MP3_HEADER_SIZE];
	unsigned char frame[MAX_MP3_FRAME_LENGTH];
	long frame_sync;
	int version;
	int layer;
	int protection;
	int bitrate;
	int samp_rate;
	int padding;
	int priv;
	int ch_mode;
	int mode_ext;
	int copyright;
	int original;
	int emphasis;
	int f_length;
}mp3_frame;

int bitRates[] = {
    0 ,  0 ,  0 ,  0 ,  0 , 0 ,
   32 , 32 , 32 , 32 , 32 , 8 ,
   64 , 48 , 40 , 48 , 16 , 16,
   96 , 56 , 48 , 56 , 24 , 24,
  128 , 64 , 56 , 64 , 32 , 32,
  160 , 80 , 64 , 80 , 40 , 40,
  192 , 96 , 80 , 96 , 48 , 48,
  224 ,112 , 96 ,112 , 56 , 56,
  256 ,128 ,112 ,128 , 64 , 64,
  288 ,160 ,128 ,144 , 80 , 80,
  320 ,192 ,160 ,160 , 96 , 96,
  352 ,224 ,192 ,176 ,112 ,112,
  384 ,256 ,224 ,192 ,128 ,128,
  416 ,320 ,256 ,224 ,144 ,144,
  448 ,384 ,320 ,256 ,160 ,160,
    0 ,  0 ,  0 ,  0 ,  0 ,  0
};

int sampRates[] = {
  44100, 22050, 11025,
  48000, 24000, 12000,
  32000, 16000,  8000,
      0,     0,     0
};

int mpegVersions[] = { 3, 0, 2, 1 };

int mpegLayers[]   = { 0, 3, 2, 1 };

int getFrameLength(mp3_frame* hdr) {
	float tmp = 144000 * hdr->bitrate / hdr->samp_rate + hdr->padding;
	return ((int) tmp);
}

int displayMp3Header(mp3_frame* hdr) {
	printf("Frame Sync: %x\n",hdr->frame_sync);
	printf("Version:    %d\n",hdr->version);
	printf("Layer:      %d\n",hdr->layer);
	printf("Protection: %x\n",hdr->protection);
	printf("Bitrate:    %d\n",hdr->bitrate);
	printf("Samp rate:  %d\n",hdr->samp_rate);
	printf("Padding:    %x\n",hdr->padding);
	printf("Private:    %x\n",hdr->priv);
	printf("Ch mode:    %x\n",hdr->ch_mode);
	printf("Mode ext:   %x\n",hdr->mode_ext);
	printf("Copyright:  %x\n",hdr->copyright);
	printf("Original:   %x\n",hdr->original);
	printf("Emphasis:   %x\n",hdr->emphasis);
}

int displayMp3HeaderSmall(mp3_frame* hdr) {
	fprintf(stderr,"%x ",hdr->frame_sync);
	fprintf(stderr,"V%d-",hdr->version);
	fprintf(stderr,"L%d ",hdr->layer);
	fprintf(stderr,"%x ",hdr->protection);
	fprintf(stderr,"%dKbps ",hdr->bitrate);
	fprintf(stderr,"%dHz ",hdr->samp_rate);
	fprintf(stderr,"%x ",hdr->padding);
	fprintf(stderr,"%x ",hdr->priv);
	fprintf(stderr,"%x ",hdr->ch_mode);
	fprintf(stderr,"%x ",hdr->mode_ext);
	fprintf(stderr,"%x ",hdr->copyright);
	fprintf(stderr,"%x ",hdr->original);
	fprintf(stderr,"%x ",hdr->emphasis);
	fprintf(stderr,"%dbytes \n",hdr->f_length);
}

int findNextMp3Frame(FILE* FH, mp3_frame* hdr) {
	int i = 0;
	int n_frame;
	long header1, header2;
	unsigned char buf1, buf2, buf3, buf4;
	while (fread(&buf1,1,1,FH) != 0) {
/*		fprintf(stderr,"BONG1\n");*/
		if ( buf1 == 0xff ) {
			fread(&buf2,1,1,FH);
			if ( (buf2&0xf0) == 0xf0) {
/*				fprintf(stderr,"BONG3\n");*/
				header1 = (((long)buf1)*256) + ((long)buf2);
				hdr->frame_sync = ((header1&0xffe0)>>5);
				hdr->version    = mpegVersions[((header1&0x001a)>>3)];
				hdr->layer      = mpegLayers[((header1&0x0006)>>1)];
				hdr->protection = ((header1&0x0001)>>0);
				fread(&buf3,1,1,FH);
				fread(&buf4,1,1,FH);
				header2 = (((long)buf3)*256) + ((long)buf4);
				hdr->bitrate    = bitRates[(hdr->version-1)*3+(hdr->layer-1)+6*((header2&0xf000)>>12)];
				hdr->samp_rate  = sampRates[(hdr->version-1)+3*((header2&0x0b00)>>10)];
				hdr->padding    = ((header2&0x0200)>>9);
				hdr->priv       = ((header2&0x0100)>>8);
				hdr->ch_mode    = ((header2&0x00b0)>>6);
				hdr->mode_ext   = ((header2&0x0030)>>4);
				hdr->copyright  = ((header2&0x000a)>>3);
				hdr->original   = ((header2&0x0008)>>2);
				hdr->emphasis   = ((header2&0x0003)>>0);
				hdr->f_length   = getFrameLength(hdr);
				hdr->data[0] = buf1;
				hdr->data[1] = buf2;
				hdr->data[2] = buf3;
				hdr->data[3] = buf4;
				if (WARNINGS)
					if (i!=0) {
						fprintf(stderr,"Jumped %d bytes.\n",i);
						displayMp3HeaderSmall(hdr);
					}
				if ( (hdr->f_length > MAX_MP3_FRAME_LENGTH) || (hdr->f_length < MIN_MP3_FRAME_LENGTH) ){
/*					fprintf(stderr,"HEY %d\n",hdr->f_length);*/
					continue;
				}
				n_frame = fread(hdr->frame,sizeof(unsigned char),(hdr->f_length-MP3_HEADER_SIZE),FH);
/*				if (n_frame!=(hdr->f_length-MP3_HEADER_SIZE))
					continue;*/
/*				fprintf(stderr,"%d %d\n",hdr->f_length,n_frame);*/
/*				fprintf(stderr,"Reading frame %d/%dbytes\n",n_frame,(hdr->f_length-MP3_HEADER_SIZE));*/
				return(0);
			}
		}
		else i++;
	}
	return(1);
}

int outputFrame(mp3_frame* hdr) {
	int bytesSent;
	char buffer[MAX_MP3_FRAME_LENGTH];
	int size;
	memcpy(buffer,hdr->data,MP3_HEADER_SIZE);
	memcpy(buffer+MP3_HEADER_SIZE,hdr->frame,hdr->f_length);
	size = hdr->f_length;
	write(1,buffer,size);
	return (1);
}
